**********************************
**********************************
** ML Nigeria
** A4 - Prediction OLS
**********************************
**********************************

/*
	READ ME
	Turn on/off: set directory line 22
		groups: lines 42-44
		outcomes: lines 46-48
		then run the desired specification
*/

capture log close
clear all
set more off
version 14.1 

*set directory
cd 
*load data
use "Data\MLNigeria_masterdata.dta", clear

*Note:
*Firms in control group who actually recived award (9 firms) dropped during data cleaning (dofile A1)
*Firms without business plan scores from RD sample dropped during data cleaning (dofile A1)
*Firms disqualified dropped during data cleaning (dofile A1)
	
*set scheme graphs
set scheme s1mono	
	
**************************************
*Data Preparation
**************************************

	**************************************
	*Keep only relevant groups
	**************************************

		*winners
		keep if (group==1|group==3)
		*non-winners
		*keep if (group==2|group==4)
		
		*global depvar o_t_totalemp1_IHS
		*global depvar o_t_sales_IHS
		global depvar o_t_profits_IHS
		keep if $depvar!=.
		sum $depvar

	**************************************
	*Global
	**************************************
	
		*Note: since we dropped the RD sample without business scores, there are no missings for these variables
		global BSCORE		b_maskill b_artic b_cap b_finsour b_finsust b_finviab b_jobcreate b_risk b_time b_unders 
		
		*Group 1&3: control for winning status (p_group1) only when using business plan scores as predictors
		*Group 2&4: Do NOT control for RD group
		*global GROUP		existing /*p_group1*/ 			
		global GROUP		existing /*p_group4*/	

		global MCKENZIE 	p_higheduc p_educ_m ///
							p_workabroad /*p_workabroad_m never missing*/ ///
							p_lot901 /*p_lot905 never missing*/ ///
							p_wealthindex /*p_wealthindex_m never missing*/ ///
							p_grit p_grit_m ///
							p_ability p_ability_m ///
							p_sector31 /*agriculture*/ p_sector19 /*IT*/
							
		global FAFCHAMPS	p_selfconfw1 p_selfconfw1_m ///
							p_motivation /*p_motivation_m never missing*/ ///
							p_satisf p_satisf_m ///
							p_satexp  p_satexp_m ///
							p_abilityfw p_abilityfw_m ///
							p_creditfw p_creditfw_m ///
							p_retail 
							
		global REGION		p_region1 p_region2 p_region3 p_region4 p_region5
		global DEMOG		p_female ///
							p_age /*p_age_m never missing*/
		global COMMON		$REGION $DEMOG ///
							p_manuf p_sector1 /*missing sector*/
							
		*w/o gender
		global DEMOG2		/*p_female*/ p_age /*p_age_m never missing*/
		global COMMON2		$REGION $DEMOG2 ///
							p_manuf p_sector1 /*missing sector*/
		*w/o age
		global DEMOG3		p_female /*p_age*/ /*p_age_m never missing*/
		global COMMON3		$REGION $DEMOG3 ///
							p_manuf p_sector1 /*missing sector*/

		*w/o age or gender
		global COMMON4		$REGION ///
							p_manuf p_sector1 /*missing sector*/							
							
	**************************************
	*Divide sample in trainig and hold-out samples
	**************************************	
	
		*sort data in random order
		set seed 1234
		gen random = uniform()
		sort random
		
		*Divide in 2 groups
		local size = _N * 0.8	
		qui gen train= (_n<=`size')
		qui gen holdout = (_n>`size')
		*tab train holdout, m col

	**************************************
	* Post-estimation program for MSE and r2
	**************************************	
	
		capture program drop bootmse
		program bootmse, rclass
			qui egen SSR = total(residual2)
			qui sum SSR
			local ssr = r(mean)
			qui count 
			local nobs = r(N)
			return scalar mse= `ssr' / `nobs'
			qui drop SSR
			macro drop ssr nobs
		end
		
		capture program drop pmse
		program pmse
		
			di "The train R^2 is " round(e(r2)*100, 0.1)
			
			*predict yhat
			qui predict yhat
			sum yhat
			
			*check if some observations below min depvar (usually 0). If so, impute them.
			qui sum $depvar if train==1
			replace yhat = r(min) if yhat<r(min)
			*check for extremely abnormal predictions
			qui sum $depvar if train==1
			local yhatmax = 2*r(max)
			sum yhat
			if r(max)>`yhatmax' {
				replace yhat = `yhatmax' if yhat>`yhatmax'
				sum yhat
				di ""
				di "NOTE: Prediction outliers imputed"
				di ""
				}						
			
			*gen residual^2
			qui gen residual2 = ($depvar - yhat)^2
			
			*number observations
			qui count if train==1 & $depvar!=.
			local train_n = r(N)
			qui count if holdout==1 & $depvar!=.
			local holdout_n = r(N)
				
			*train MSE	
			qui egen train_SSR = total(residual2) if !missing($depvar) & train==1
			qui sum train_SSR
			local train_mse= r(mean) / `train_n'
			di "The train MSE is " round(`train_mse', 0.01)

			*train r2
			qui corr $depvar yhat if train==1
			local train_r2 = r(rho)^2
			di "The new train R^2 is " round(`train_r2'*100, 0.1)
			
			*holdout r2
			qui corr $depvar yhat if holdout==1
			local holdout_r2 = r(rho)^2
			di "The holdout R^2 is " round(`holdout_r2'*100, 0.1)
			
			*holdout MSE	
			qui egen holdout_SSR = total(residual2) if !missing($depvar) & holdout==1
			qui sum holdout_SSR
			local holdout_mse= r(mean) / `holdout_n'
			di "The holdout MSE is " round(`holdout_mse', 0.01)
			
			preserve
				keep if holdout==1
				*check thta bootmse provides the same average mse
					*bootmse
					*return list
				bootstrap r(mse), reps(1000) seed(1111): bootmse
			restore
			
			drop yhat residual2 train_SSR holdout_SSR
			macro drop train_n holdout_n train_mse holdout_r2 holdout_mse train_r2
		
		end
		
exit		
		
**************************************
* Prediction
**************************************

	**************************************
	*Checks
	**************************************

		regress $depvar b_totmarkf $GROUP if train==1
		predict yhat
		di e(rmse)^2
		
		*check that in-sample r2 is consistent
		di e(r2)
		qui corr $depvar yhat if train==1
		di r(rho)^2
		
		*check that in-sample MSE is consistent
		qui gen residual2 = ($depvar - yhat)^2
		qui egen SSR = total(residual2) if !missing($depvar) & train==1
		sum SSR
		local ssr = r(mean)
		di `ssr'
		qui count if train==1 & $depvar!=.
		local train_n = r(N)	
		di `train_n'
		local train_mse= `ssr' / `train_n'
		di `train_mse'
		*Note: different because Stata divides by n-k, while the standard definition is "divide by n"
		local train_mse_b= `ssr' / (`train_n' - e(rank))
		di `train_mse_b'
		
	**************************************
	*Constant
	**************************************
	
		regress $depvar if train==1
		pmse	
		
	**************************************
	*Initial Score
	**************************************

		*no missings
		*codebook totalmark
		
		*non-winners
		regress $depvar b_totmarkf $GROUP if train==1
		pmse	
		
		*winners
		regress $depvar b_totmarkf $GROUP b_totaw b_totaw_m if train==1
		pmse			
		
	**************************************
	*Business Plan Total Score
	**************************************

		*no missings
		*codebook totalmarks
		
		*Non-winners
		regress $depvar b_totmark $GROUP if train==1
		pmse			

		*Winners
		*Control also for winning status since function of business scores
		regress $depvar b_totmark $GROUP p_group1 b_totaw b_totaw_m if train==1
		pmse
		
		*********************
		*Change sample size (for non-winners)
		*********************
		
			*run same regression with different sample size 
			gen tmse = .
			gen tr2 =.
			gen hmse = .
			gen hr2 =.
			gen tsize=.
			foreach i of numlist 1(1)10{
			
				qui sort random, stable
				local tsize = (_N * 0.8) * (0.1*`i')
				qui replace train = (_n<=`tsize')
				qui count if train==1
				qui replace tsize = r(N) in `i'
				
				regress $depvar b_totmark $GROUP if train==1
							
				*predict yhat
				qui predict yhat			
				*check if some observations below min depvar (usually 0). If so, impute them.
				qui sum $depvar if train==1
				replace yhat = r(min) if yhat<r(min)
				sum yhat
				*check for extremely abnormal predictions
				qui sum $depvar if train==1
				local yhatmax = 2*r(max)
				replace yhat = `yhatmax' if yhat>`yhatmax'
				sum yhat				
				*gen residual^2
				qui gen residual2 = ($depvar - yhat)^2			
				*number observations
				qui count if train==1 & $depvar!=.
				local train_n = r(N)
				qui count if holdout==1 & $depvar!=.
				local holdout_n = r(N)
					
				*train MSE	
				qui egen train_SSR = total(residual2) if !missing($depvar) & train==1
				qui sum train_SSR
				local train_mse= r(mean) / `train_n'
				*di "The train MSE is " round(`train_mse', 0.01)
				qui replace tmse = `train_mse' in `i'			
				*train r2
				qui corr $depvar yhat if train==1
				local train_r2 = r(rho)^2
				*di "The new train R^2 is " round(`train_r2'*100, 0.1)
				qui replace tr2 = `train_r2'*100 in `i'
				
				*holdout MSE	
				qui egen holdout_SSR = total(residual2) if !missing($depvar) & holdout==1
				qui sum holdout_SSR
				local holdout_mse= r(mean) / `holdout_n'
				*di "The holdout MSE is " round(`holdout_mse', 0.01)
				qui replace hmse = `holdout_mse' in `i'
				*holdout r2
				qui corr $depvar yhat if holdout==1
				local holdout_r2 = r(rho)^2
				*di "The holdout R^2 is " round(`holdout_r2'*100, 0.1)
				qui replace hr2 = `holdout_r2'*100 in `i'
				
				drop yhat residual2 train_SSR holdout_SSR
				macro drop train_n holdout_n train_mse holdout_r2 holdout_mse train_r2
			}
			
			*see data
			gen size = _n*10 in 1/10
			list tmse tr2 hmse hr2 tsize size in 1/10
			
			*plot in-sample and out-of-sample performances
			twoway scatter tmse size ///
				,  	title("In-sample MSE") xtitle("%Train sample") ytitle("MSE") ///
					connect(l) saving("Output/FigA4a.gph", replace)
			twoway scatter hmse size ///
				,  	title("Out-of-sample MSE") xtitle("%Train sample") ytitle("MSE") ///
					connect(l) saving("Output/FigA4b.gph", replace)
			twoway scatter tr2 size ///
				,  	title("In-sample R2") xtitle("%Train sample") ytitle("R2") ///
					connect(l) saving("Output/FigA4c.gph", replace)
			twoway scatter hr2 size ///
				,  	title("Out-of-sample R2") xtitle("%Train sample") ytitle("R2") ///
					connect(l) saving("Output/FigA4d.gph", replace)
			*combine into one figure for paper
			graph combine "Output/FigA4a.gph" "Output/FigA4b.gph" "Output/FigA4c.gph" "Output/FigA4d.gph", ///
					col(2) title("Total employment among non-winners") subtitle("Predictive power business scores by varying sample size") ///
					saving("Output/FigureA4.gph", replace)
			*only MSE		
			graph combine "Output/FigA4a.gph" "Output/FigA4b.gph", ///
					ycommon col(2) title("Total employment among non-winners") subtitle("Predictive power business scores by varying sample size") ///
					saving("Output/FigureA4.gph", replace)
					
					
	**************************************
	*Business Plan Component Score
	**************************************
	
		*Non-winners
		regress $depvar $BSCORE $GROUP if train==1
		pmse
		
		*Winners
		*Control also for winning status since function of business scores
		regress $depvar $BSCORE $GROUP p_group1 b_totaw b_totaw_m if train==1
		pmse
	
	**************************************
	*Gender
	**************************************
	
		*no missings
		*codebook sex
		regress $depvar p_female $GROUP if train==1
		pmse			
		
		*with winning amount
		regress $depvar p_female $GROUP b_totaw b_totaw_m if train==1
		pmse			
					
	**************************************
	*Age
	**************************************

		*no missings
		*codebook age
		regress $depvar p_age $GROUP if train==1
		pmse	
		
		*with winning amount
		regress $depvar p_age $GROUP b_totaw b_totaw_m if train==1
		pmse	
				
	**************************************
	*Necessity Entrepreneurship
	**************************************

		*no missings
		*codebook rate_employment
		regress $depvar p_needjob $GROUP if train==1
		pmse				

		*with winning amount
		regress $depvar p_needjob $GROUP b_totaw b_totaw_m if train==1
		pmse	
		
	**************************************
	*Grit
	**************************************

		regress $depvar p_grit p_grit_m $GROUP if train==1
		pmse	

		*with winning amount
		regress $depvar p_grit p_grit_m $GROUP b_totaw b_totaw_m if train==1
		pmse			
					
	**************************************
	*Backward Digit-span Recall
	**************************************
	
		regress $depvar p_digitspan p_digitspan_m $GROUP if train==1
		pmse	

		*with winning amount
		regress $depvar p_digitspan p_digitspan_m $GROUP b_totaw b_totaw_m if train==1
		pmse			
						
	**************************************
	*Raven Test
	**************************************

		regress $depvar p_raven p_raven_m $GROUP if train==1
		pmse	

		*with winning amount
		regress $depvar p_raven p_raven_m $GROUP b_totaw b_totaw_m if train==1
		pmse			
				
	**************************************
	*Registration time
	**************************************

		*no missings
		*codebook reg_time
		regress $depvar p_lateentrant $GROUP if train==1
		pmse		

		*with winning amount
		regress $depvar p_lateentrant $GROUP b_totaw b_totaw_m if train==1
		pmse			
		
	**************************************
	*McKenzie
	**************************************
	
		regress $depvar $COMMON $MCKENZIE $GROUP if train==1
		pmse	

		*with winning amount
		regress $depvar $COMMON $MCKENZIE $GROUP b_totaw b_totaw_m if train==1
		pmse			

		*w/o gender
		regress $depvar $COMMON2 $MCKENZIE $GROUP if train==1
		pmse	

		*w/o age
		regress $depvar $COMMON3 $MCKENZIE $GROUP if train==1
		pmse	

		*w/o age or gender
		regress $depvar $COMMON4 $MCKENZIE $GROUP if train==1
		pmse			
		
	**************************************
	*Fafchamps-Woodruff
	**************************************

		regress $depvar $COMMON $FAFCHAMPS $GROUP if train==1
		pmse				

		*with winning amount
		regress $depvar $COMMON $FAFCHAMPS $GROUP b_totaw b_totaw_m if train==1
		pmse					
		
		*plus business scores
		*Non-winners
		regress $depvar $COMMON $FAFCHAMPS $GROUP b_totmark if train==1
		pmse
		*Winners
		regress $depvar $COMMON $FAFCHAMPS $GROUP b_totmark p_group1 b_totaw b_totaw_m if train==1
		pmse
		
		*add baseline employment as regressor 
		egen baseline_empl = rowtotal(	workers_with_no_qualifications workers_with_primary_education ///
										workers_with_secondary_education workers_with_tertiary_education) 
		gen baseline_empl_m = (	workers_with_no_qualifications==. & workers_with_primary_education==. ///
								& workers_with_secondary_education==. & workers_with_tertiary_education==.)
		regress $depvar $COMMON $FAFCHAMPS $GROUP baseline_empl baseline_empl_m if train==1
		pmse				
			
		*w/o gender	
		regress $depvar $COMMON2 $FAFCHAMPS $GROUP if train==1
		pmse				

		*w/o age
		regress $depvar $COMMON3 $FAFCHAMPS $GROUP if train==1
		pmse						

		*w/o age or gender
		regress $depvar $COMMON4 $FAFCHAMPS $GROUP if train==1
		pmse						
		
exit
