**********************************
**********************************
** ML Nigeria
** B2 - Prediction LASSO
**********************************
**********************************

/*
	READ ME
	Turn on/off or rename: 
		set directory: line 31 
		log file line 35
		groups: lines 50-53
		outcomes: lines 55-57
		reduce sample size (robustness check): 75-79
		change set of inputs: age/gender (robustness check): 162-166
		add award amount (robustness check for winners): 134, 175
		add business scores (robustness check): 136-137, 177-178
		option collinear (robustness check): 253, 322
		predictions: 400-402
		predictors: 414-415
*/

capture log close
clear all
set more off
set matsize 800
version 14.1 

*set directory
cd 
*load data
use "Data\MLNigeria_masterdata.dta", clear
*log file
log using "Output\Predict_LASSO_EmplNoWin", replace

*Note:
*Firms in control group who actually recived award (9 firms) dropped during data cleaning (dofile A1)
*Firms without business plan scores from RD sample dropped during data cleaning (dofile A1)
*Firms disqualified dropped during data cleaning (dofile A1)

**************************************
*Data Preparation
**************************************

	**************************************
	*Keep only relevant groups
	**************************************

		*winners
		*keep if (group==1|group==3)
		*non-winners
		keep if (group==2|group==4)
		
		global depvar o_t_totalemp1_IHS
		*global depvar o_t_sales_IHS
		*global depvar o_t_profits_IHS
		keep if $depvar!=.

	**************************************
	*Divide sample in trainig and hold-out samples
	**************************************	
	
		*sort data in random order
		set seed 1234
		gen random = uniform()
		sort random
		
		*Divide in 2 groups
		local size = _N * 0.8	
		qui gen train = (_n<=`size')
		qui gen holdout = (_n>`size')
		*tab train holdout, m col
		
		*robustness check: reduce sample size
		*local tsize = (_N * 0.8) * 0.8
		*qui replace train = (_n<=`tsize')
		*tab train holdout, m col
		*observations not used are zero for both train and holdout
		
	**************************************
	*Divide training sample in 5 folds
	**************************************

		************************
		*Divide training sample in 5 groups
		************************	
	
			count if train==1
			local size2 = `r(N)'/5

			qui generate foldc=0 if train==1
			qui replace foldc=1 if _n>`size2' & train==1
			qui replace foldc=2 if _n>`size2'*2 & train==1
			qui replace foldc=3 if _n>`size2'*3 & train==1
			qui replace foldc=4 if _n>`size2'*4 & train==1

			tab foldc, gen(fold)
			*tab foldc train, m
			*holdout sample not in  the 5 folds, as expected
			*tab foldc holdout, m

		************************
		*Folds
		************************
		
			*Fold A
			gen byte trainA = (fold2==1 | fold3==1 | fold4==1 | fold5==1)
			gen byte testA = (fold1==1)
		
			*FoldB
			gen byte trainB = (fold1==1| fold3==1 | fold4==1 | fold5==1)
			gen byte testB = (fold2==1)
			
			*FoldC
			gen byte trainC = (fold1==1 | fold2==1| fold4==1 | fold5==1)
			gen byte testC = (fold3==1)
			
			*Fold D
			gen byte trainD = (fold1==1 | fold2==1 | fold3==1| fold5==1)
			gen byte testD = (fold4==1)
			
			*Fold E
			gen byte trainE = (fold1==1 | fold2==1 | fold3==1 | fold4==1)
			gen byte testE = (fold5==1)
		
	**************************************
	*Drop predictors that are always zero
	**************************************
	
		unab pred0: p_*
		global PRED0	`pred0'
		*robustness check: add award amount
		*global PRED0	`pred0' b_totaw b_totaw_m
*		*NOTE: FOR THE MAIN TABLE, DO NOT INCLUDE BUSINESS SCORES IN THE INPUTS
		*unab bus0: b_*
		*global PRED0	`pred0' `bus0'
		
		foreach var of global PRED0 {
			qui sum `var' if train==1 
			if r(mean)==0 & r(sd)==0 {
				di "`var'"
				drop `var'
			}	
		}

		*FOR GROUP 1 AND 3. NOTE: FOR THE MAIN TABLE, DO NOT INCLUDE BUSINESS SCORES IN THE INPUTS
		*drop b_round1 since it is always one (all winners passed the first round)
		*drop b_round1. Not necessary since not generated in new version data. 
		*all 1 since we have dropped RD sample with no business scores
		
		*FOR GROUP 2 AND 4
		*drop also missing variable indicators related to these variables (only for bussiness name, the others were created from tab, gen so they are fine)
		*already dropped in replication file
		*drop p_bussn_m p_bussnb_m p_bussnc_m p_mask_m p_mgive_m 
		*drop b_totaw_m

	**************************************
	* Robustness checks: drop certain inputs
	**************************************	
		
		*drop gender
		*drop p_female
		
		*drop age
		*drop p_age
		
	**************************************
	*Global
	**************************************
	
		unab individualpredictors: p_*
		global PREDICTOR 	`individualpredictors' existing
		*robustness check: add award amount
		*global PREDICTOR 	`individualpredictors' existing b_totaw b_totaw_m
*		*NOTE: FOR THE MAIN TABLE, DO NOT INCLUDE BUSINESS SCORES IN THE INPUTS
		*unab businessplan: b_*
		*global PREDICTOR 	`individualpredictors' `businessplan' existing
				
	**************************************
	*Keep only relevant variables
	**************************************		
		
		keep uid group o_* $PREDICTOR random train* test* holdout 
		qui compress
		*count predictors
		unab countvars : $PREDICTOR
		di `: word count `countvars''
		
		*Following Mullainathan & Spiess (2017), do not drop collinear variables
		*qui _rmcoll $PREDICTOR, forcedrop
		*global PREDICTOR `r(varlist)'
				
	**************************************
	*Scale Regressors
	**************************************	
	
		*get min and max from the train sample only (as stressed in Mullainathan & Spiess, 2017)
		foreach var of global PREDICTOR {
			*Note: this do not change dummy variables: you subtract zero and divide by 1
			qui sum `var' if train==1
			qui replace `var' = (`var'-r(min))/(r(max)-r(min))
			}
				
	**************************************
	* Check no missing values
	**************************************	
	
		*check all have same number of obs as depvar
		qui sum $depvar if train==1
		local Ncheck = r(N)
		
		foreach var of global PREDICTOR {
			qui sum `var' if train==1
			if r(N)!=`Ncheck' {
				di "`var'"
				_exit
				}
		}	
			
*exit	
		
****************************************************************************************************************************************************************		
****************************************************************************************************************************************************************		
****************************************************************************************************************************************************************		

**************************************
* LASSO
**************************************

	*sample size
	qui sum $depvar
	local Ncheck = r(N)	
	*Choose best lambda with grid-search 
	local minerr=.
	local maxnimputed = 0
	foreach tlambda of numlist 0 1e-12 1e-11 5e-11 1e-10 5e-10 1e-09 5e-09 1e-08 5e-08 ///
                            1e-07 2e-07 3e-07 4e-07 5e-07 6e-07 7e-07 8e-07 9e-07 ///
							1e-06 2e-06 3e-06 4e-06 5e-06 6e-06 7e-06 8e-06 9e-06 ///
							1e-05 2e-05 3e-05 4e-05 5e-05 6e-05 7e-05 8e-05 9e-05 ///
							1e-04 2e-04 3e-04 4e-04 5e-04 6e-04 7e-04 8e-04 9e-04 ///
							1e-03 2e-03 3e-03 4e-03 5e-03 6e-03 7e-03 8e-03 9e-03 ///
							1e-02 2e-02 3e-02 4e-02 5e-02 6e-02 7e-02 8e-02 9e-02 ///
							1e-01 5e-01 1 5 10 50 {
		global gtest_mse = 0					   
		foreach fold in A B C D E {
			
			*LASSO
			di "Fold:`fold'"
			display "Lambda=`tlambda'"
			sort random, stable
			*Robustness check: similar results with or without the option collinear  
			qui lassoregress $depvar $PREDICTOR if train`fold'==1, lambda(`tlambda') /*collinear*/
			
			*prediction
			qui predict yhat
			
			*check same number of obs
			sum yhat
			if r(N)!=`Ncheck' {
					_exit
				}
			*check if some observations below min depvar (usually 0). If so, impute them.
			qui sum $depvar if train`fold'==1
			qui replace yhat = r(min) if yhat<r(min)
			*check for extremely abnormal predictions
			qui sum $depvar if train`fold'==1
			local yhatmax = 2*r(max)
			sum yhat
			if r(max)>`yhatmax' {
					count if yhat>`yhatmax'
					if r(N)>`maxnimputed' {
						local maxnimputed = r(N)
						di "Max number of positive predictions imputed " `maxnimputed'
						}
					replace yhat = `yhatmax' if yhat>`yhatmax'
					sum yhat
					di ""
					di "NOTE: Prediction outliers imputed"
					di ""
				}						
			
			*SSR
			qui gen residual2 = ($depvar - yhat)^2
			qui egen SSR = total(residual2) if !missing($depvar) & test`fold'==1
			qui sum SSR
			local ssr = r(mean)
			*MSE
			qui count if !missing($depvar) & test`fold'==1
			local test_n = r(N)
			local test_mse= `ssr' / `test_n'
			di "MSE in test sample in fold `fold' for Lambda=`tlambda' is " `test_mse'
			global gtest_mse = $gtest_mse + `test_mse'
			
			*drop variables
			qui drop yhat residual2 SSR
			qui macro drop ssr test_n test_mse 

			}
			
			global gtest_mse = $gtest_mse / 5
			di "The average MSE for Lambda=`tlambda' in the 5-fold CV is " round($gtest_mse, 0.1)
			
			*min mse
			if $gtest_mse<`minerr' {
				local minerr= $gtest_mse
				display "The new lowest CV MSE rate is " `minerr'
				local bestLambda = `tlambda'
				display "Current best values: Lambda=`bestLambda'"
				}
				
	}	

	*check imputation not pervasive
	di "Max number of positive predictions imputed " `maxnimputed'
	
	*Best Lambda
	display "Chosen starting values: Lambda=`bestLambda'" 
	
	*LASSO with best lambda
	sort random, stable
	qui lassoregress $depvar $PREDICTOR if train==1, lambda(`bestLambda') /*collinear*/
	
	*selected predictors
	global lassoSel `e(varlist_nonzero)'
	di e(varlist_nonzero)
	
	*predictions
	qui predict yhat
	sum yhat
	
	*check if some observations below min depvar (usually 0). If so, impute them.
	qui sum $depvar if train==1
	replace yhat = r(min) if yhat<r(min)
	*check for extremely abnormal predictions
	qui sum $depvar if train==1
	local yhatmax = 2*r(max)
	sum yhat
	if r(max)>`yhatmax' {
		replace yhat = `yhatmax' if yhat>`yhatmax'
		sum yhat
		di ""
		di "NOTE: Prediction outliers imputed"
		di ""
		}						

	*gen residual^2
	qui gen residual2 = ($depvar - yhat)^2
	
	*Train
	*SSR	
	qui egen SSRtrain = total(residual2) if !missing($depvar) & train==1
	qui sum SSRtrain
	local ssr_train = r(mean)
	*MSE
	qui count if !missing($depvar) & train==1
	local train_n = r(N)
	local train_mse= `ssr_train' / `train_n'
	di "MSE in training sample is " `train_mse'
	*R2
	qui corr $depvar yhat if train==1
	local train_r2 = r(rho)^2
	di "The train R^2 is " `train_r2'
		
	*Hold-out sample
	qui egen SSRholdout = total(residual2) if !missing($depvar) & holdout==1
	qui sum SSRholdout
	local ssr_holdout = r(mean)
	*MSE
	qui count if !missing($depvar) & holdout==1
	local holdout_n = r(N)
	local holdout_mse= `ssr_holdout' / `holdout_n'
	di "MSE in holdout sample is " `holdout_mse'
	*R2
	qui corr $depvar yhat if holdout==1
	local holdout_r2 = r(rho)^2
	di "The holdout R^2 is " `holdout_r2'
	
	*boostrap out-of-sample performance
	capture program drop bootmse
	program bootmse, rclass
		qui egen SSR = total(residual2)
		qui sum SSR
		local ssr = r(mean)
		qui count 
		local nobs = r(N)
		return scalar mse= `ssr' / `nobs'
		qui drop SSR
		macro drop ssr nobs
	end
	preserve
		keep if holdout==1 
		*check thta bootmse provides the same average mse
			*bootmse
			*return list
		bootstrap r(mse), reps(1000) seed(1111): bootmse
	restore
	
	*save dataset 	
	rename yhat LASSO_Empl24
	save "LASSOEmpl24.dta", replace
	
	*Variables selected
	gen str1 predictors= ""
	local nxx : word count $lassoSel
	forvalues ii = 1/`nxx' { 
		local temp1 : word `ii' of $lassoSel
		label var `temp1' `temp1'
		local lab: variable label `temp1'
		replace predictors = "`lab'" in `ii'
	} 
	keep predictors
	drop if predictors==""
	gen E24_varsel = 1
	save "LASSOselE24.dta", replace	

log close	
exit
