**********************************
**********************************
** ML Nigeria
** B3b - Prediction Boosting Binary
**********************************
**********************************

/*
	READ ME
	Turn on/off: set directory line 25
		log file name: line 29
		Only FaWo input (robustness check): lines 146-160
		load boost plug-in: line 216
		predictions: lines 305-308
		influence: 334-335
*/

capture log close
clear all
set more off
set matsize 800
version 14.1 

*set directory
cd 
*load data
use "Data\MLNigeria_masterdata.dta", clear
*log file
log using "Output\Predict_Boost_Surv", replace

*Note:
*Firms in control group who actually recived award (9 firms) dropped during data cleaning (dofile A1)
*Firms without business plan scores from RD sample dropped during data cleaning (dofile A1)
*Firms disqualified dropped during data cleaning (dofile A1)

**************************************
*Data Preparation
**************************************

	**************************************
	*Keep only relevant groups
	**************************************

		*non-winners
		keep if (group==2|group==4)
		
		global depvar o_t_operatefirm
		keep if $depvar!=.

	**************************************
	*Divide sample in trainig and hold-out samples
	**************************************	
	
		*sort data in random order
		set seed 1234
		gen random = uniform()
		sort random
		
		*Divide in 2 groups
		local size = _N * 0.8	
		qui gen train= (_n<=`size')
		qui gen holdout = (_n>`size')
		*tab train holdout, m col
		
	**************************************
	*Divide training sample in 5 folds
	**************************************

		************************
		*Divide training sample in 5 groups
		************************	
	
			count if train==1
			local size2 = `r(N)'/5

			qui generate foldc=0 if train==1
			qui replace foldc=1 if _n>`size2' & train==1
			qui replace foldc=2 if _n>`size2'*2 & train==1
			qui replace foldc=3 if _n>`size2'*3 & train==1
			qui replace foldc=4 if _n>`size2'*4 & train==1

			tab foldc, gen(fold)
			*tab foldc train, m
			*holdout sample not in  the 5 folds, as expected
			*tab foldc holdout, m

		************************
		*Folds
		************************
		
			*Fold A
			gen byte trainA = (fold2==1 | fold3==1 | fold4==1 | fold5==1)
			gen byte testA = (fold1==1)
		
			*FoldB
			gen byte trainB = (fold1==1| fold3==1 | fold4==1 | fold5==1)
			gen byte testB = (fold2==1)
			
			*FoldC
			gen byte trainC = (fold1==1 | fold2==1| fold4==1 | fold5==1)
			gen byte testC = (fold3==1)
			
			*Fold D
			gen byte trainD = (fold1==1 | fold2==1 | fold3==1| fold5==1)
			gen byte testD = (fold4==1)
			
			*Fold E
			gen byte trainE = (fold1==1 | fold2==1 | fold3==1 | fold4==1)
			gen byte testE = (fold5==1)
		
	**************************************
	*Drop predictors that are always zero
	**************************************
	
		unab pred0: p_*
		global PRED0	`pred0'
*		*NOTE: FOR THE MAIN TABLE, DO NOT INCLUDE BUSINESS SCORES IN THE INPUTS
		*unab bus0: b_*
		*global PRED0	`pred0' `bus0'
		
		foreach var of global PRED0 {
			qui sum `var' if train==1 
			if r(mean)==0 & r(sd)==0 {
				di "`var'"
				drop `var'
			}	
		}
	
		*FOR GROUP 2 AND 4
		*drop also missing variable indicators related to these variables (only for bussiness name, the others were created from tab, gen so they are fine)
		*already dropped in replication file
		*drop p_bussn_m p_bussnb_m p_bussnc_m p_mask_m p_mgive_m 
		*drop b_totaw_m

	**************************************
	*Global
	**************************************
	
		unab individualpredictors: p_*
		global PREDICTOR 	`individualpredictors' existing
*		*NOTE: FOR THE MAIN TABLE, DO NOT INCLUDE BUSINESS SCORES IN THE INPUTS
		*unab businessplan: b_*
		*global PREDICTOR 	`individualpredictors' `businessplan' existing

/*
		*robustness check: use only FaWo inputs
		global GROUP		existing /*p_group4*/	
		global FAFCHAMPS	p_selfconfw1 p_selfconfw1_m ///
							p_motivation /*p_motivation_m never missing*/ ///
							p_satisf p_satisf_m ///
							p_satexp  p_satexp_m ///
							p_abilityfw p_abilityfw_m ///
							p_creditfw p_creditfw_m ///
							p_retail 
		global REGION		p_region1 p_region2 p_region3 p_region4 p_region5
		global DEMOG		p_female ///
							p_age /*p_age_m never missing*/
		global COMMON		$REGION $DEMOG ///
							p_manuf p_sector1 /*missing sector*/
		global PREDICTOR $COMMON $FAFCHAMPS $GROUP
*/
				
	**************************************
	*Keep only relevant variables
	**************************************		
		
		keep uid group o_* $PREDICTOR random train* test* holdout 
		qui compress
		*count predictors
		unab countvars : $PREDICTOR
		di `: word count `countvars''
		
		*Following Mullainathan & Spiess (2017), do not drop collinear variables
		*qui _rmcoll $PREDICTOR, forcedrop
		*global PREDICTOR `r(varlist)'
				
	**************************************
	*Scale Regressors
	**************************************	
	
		*get min and max from the train sample only (as stressed in Mullainathan & Spiess, 2017)
		foreach var of global PREDICTOR {
			*Note: this do not change dummy variables: you subtract zero and divide by 1
			qui sum `var' if train==1
			qui replace `var' = (`var'-r(min))/(r(max)-r(min))
			}
				
	**************************************
	* Check no missing values
	**************************************	
	
		*check all have same number of obs as depvar
		qui sum $depvar if train==1
		local Ncheck = r(N)
		
		foreach var of global PREDICTOR {
			qui sum `var' if train==1
			if r(N)!=`Ncheck' {
				di "`var'"
				_exit
				}
		}		
	
*exit	
		
****************************************************************************************************************************************************************		
****************************************************************************************************************************************************************		
****************************************************************************************************************************************************************		

**************************************
* Boosting
**************************************

	*load pluging
	capture program boost_plugin, plugin using("C:\ado\plus\b\boost64.dll")
		
	*It is important that the observations are in random order before invoking the boost command because otherwise 
	*the test data may be different from the training data in a systematic way.
	sort random, stable

	*sample size
	qui sum $depvar
	local Ncheck = r(N)
	*Choose best parameters with grid-search 
	local minerr=0
	foreach iter of numlist 1/150 {
		foreach inter of numlist 1/8 {
			foreach nshrink of numlist 1 0.1 {
				foreach dist in normal logistic {
					global avaccu = 0					   
					foreach fold in A B C D E {
						
						*Boosting
						di "Fold:`fold'"
						di "Current parameters: NInteractions=`inter' Maxiter=`iter' Shrink = `nshrink' Distribution = `dist'"
						sort random, stable
						qui boost $depvar $PREDICTOR if train`fold'==1, ///
							distribution(`dist') maxiter(`iter') predict(probyhat) shrink(`nshrink') bag(0.5) trainfraction(1) interaction(`inter') seed(1234)
						
						*check same number of obs
						sum probyhat
						if r(N)!=`Ncheck' {
							_exit
							}
				
						*prediction
						qui gen yhat = 1 if probyhat>=0.5 & !missing(probyhat)
						qui replace yhat = 0 if probyhat<0.5 & !missing(probyhat)
						
						*accuracy
						qui generate accuracy = ($depvar == yhat) if !missing(yhat)
						qui sum accuracy if test`fold'==1 
						local accuracy= r(mean)
						di "Accuracy in test sample in fold `fold' is" `accuracy'
						global avaccu = $avaccu + `accuracy'
						
						*drop variables
						qui drop probyhat yhat accuracy
						qui macro drop accuracy 

						}
						
						global avaccu = $avaccu *100 / 5
						di "The average Accuracy in the 5-fold CV is " round($avaccu, 0.1)
						
						*max accuracy rate
						if $avaccu>`minerr' {
							local minerr= $avaccu
							display "The new highest CV accuracy rate is " `minerr'
							local bestIter = `iter'
							local bestInter = `inter'
							local bestShrink = `nshrink'
							local bestDist `dist'
							display "Current best parameters: NInteractions=`bestInter' Maxiter=`bestIter' Shrink = `nshrink' Distribution = `bestDist'"							
							}
				}	
			}
		}
	}

	*Best parameters
	display "Chosen parameters: NInteractions=`bestInter' Maxiter=`bestIter' Shrink = `bestShrink' Distribution = `bestDist'"
	sort random, stable
	qui boost $depvar $PREDICTOR if train==1, ///
		distribution(`bestDist') maxiter(`bestIter') predict(probyhat) shrink(`bestShrink') bag(0.5) trainfraction(1) interaction(`bestInter') seed(1234) 
	qui gen yhat = 1 if probyhat>=0.5 & !missing(probyhat)
	qui replace yhat = 0 if probyhat<0.5 & !missing(probyhat)	
	qui gen accuracy = ($depvar == yhat) if !missing(yhat)

	*Train
	tab $depvar yhat if train==1, row
	sum accuracy if train==1
		
	*Hold-out sample
	tab $depvar yhat if holdout==1, row
	sum accuracy if holdout==1

	*boostrap out-of-sample performance
	preserve
		keep if holdout==1 
		bootstrap r(mean), reps(1000) seed(1111): sum accuracy
	restore
	
	*save dataset 	
	rename yhat Boost_Su24
	save "BoostSu24.dta", replace
	
	drop accuracy Boost_Su24 probyhat
	
**************************************
*Influence
**************************************

	*boost
	qui sort random, stable
	qui boost $depvar $PREDICTOR if train==1, ///
		distribution(`bestDist') maxiter(`bestIter') predict(probyhat) shrink(`bestShrink') bag(0.5) trainfraction(1) interaction(`bestInter') seed(1234) influence
	
	*check same results	
	gen yhat = 1 if probyhat>=0.5 & !missing(probyhat)
	replace yhat = 0 if probyhat<0.5 & !missing(probyhat)
	gen accuracy = ($depvar == yhat) if !missing(yhat)
	sum accuracy if holdout==1	
	drop accuracy
	
	*influence
	matrix influence = e(influence)
	svmat2 influence, rnames(predictors)
	keep predictors influence1 
	drop if influence1==.
	drop if influence1==0
	
	*save influence
	rename influence1 InfluSu24
	save "InfluSu24.dta", replace

log close	
exit
