**********************************
**********************************
** ML Nigeria
** A10 - Ensemble
**********************************
**********************************

/*
	READ ME
	Turn on/off or rename: set directory line 51
		log file name: line 55
		groups: lines 71-73
		outcomes: lines 75-77
		parameters: lines 84-136
		judges 1: lines 456-458
		judges 2: lines 627-629
*/

**************************************
*How to fit ensembles (from Millainathan and Spiess, 2017)
**************************************

/*

	Step 1: We divide the training data randomly into 5 folds to perform cross-validation;
		for every fold, we fit each of the algorithms that make up the ensemble on all other folds, and	predict on the current fold.	
	Step 2: We have thus obtained one prediction per unit in the training sample for every algorithm. 
		To fit weights, we simply run a linear regression of the outcome on these predicted values in the full training sample, 
		and store the resulting linear model.	
	Step 3: To predict in the hold-out, we fit each algorithm on the full training sample, 
		obtain predictions for each algorithm on the hold-out, 
		and then predict with the linear model obtained in the previous step from these fitted values.
	
	Note that running OLS does not necessarily produce weights that add up to one, and may also roduce an intercept. 
	While we could exclude the intercept and search instead for weights that add up to one, 
	in our experience the OLS ensemble fit (with intercept) performs as well or even better	than such an ensemble.

*/

**************************************
*Load data
**************************************

capture log close
clear all
set more off
set matsize 11000
version 14.1 

*set directory
cd 
*load data
use "Data\MLNigeria_masterdata.dta", clear
*log file
log using "Output\Ensemble_ProfNoWin", replace

*Note:
*Firms in control group who actually recived award (9 firms) dropped during data cleaning (dofile A1)
*Firms without business plan scores from RD sample dropped during data cleaning (dofile A1)
*Firms disqualified dropped during data cleaning (dofile A1)

**************************************
*Data Preparation
**************************************

	**************************************
	*Keep only relevant groups
	**************************************

		*winners
		*keep if (group==1|group==3)
		*non-winners
		keep if (group==2|group==4)
		
		*global depvar o_t_totalemp1_IHS
		*global depvar o_t_sales_IHS
		global depvar o_t_profits_IHS
		keep if $depvar!=.
		
	**************************************
	*Optimal values for each outcome
	**************************************
/*
		*E24
		local bestLambda = 0.08
		local bestC = 5000
		local bestGamma = 5e-08
		local bestEps = 5e-05
		local bestIter = 94
		local bestInter = 1
		local bestShrink = 0.1	

		*E13
		local bestLambda = 0.08
		local bestC = 5e+05
		local bestGamma = 5e-09
		local bestEps = 1
		local bestIter = 19
		local bestInter = 8
		local bestShrink = 0.1

		*S24
		local bestLambda = 0.5
		local bestC = 5
		local bestGamma = 5e-04
		local bestEps = 5e-04
		local bestIter = 113
		local bestInter = 1
		local bestShrink = 0.1
		
		*S13
		local bestLambda = 0.5
		local bestC = 5
		local bestGamma = 0.05
		local bestEps = 1
		local bestIter = 13
		local bestInter = 6
		local bestShrink = 0.1	
*/		
		*P24
		local bestLambda = 0.5
		local bestC = 5
		local bestGamma = 0.05
		local bestEps = 0.5
		local bestIter = 135
		local bestInter = 1
		local bestShrink = 0.1	
/*		
		*P13
		local bestLambda = 0.5
		local bestC = 5
		local bestGamma = 0.05
		local bestEps = 1
		local bestIter = 43
		local bestInter = 1
		local bestShrink = 0.1
*/		
	**************************************
	*Divide sample in trainig and hold-out samples
	**************************************	
	
		*sort data in random order
		set seed 1234
		gen random = uniform()
		sort random
		
		*Divide in 2 groups
		local size = _N * 0.8	
		qui gen train = (_n<=`size')
		qui gen holdout = (_n>`size')
		*tab train holdout, m col
				
	**************************************
	*Divide training sample in 5 folds
	**************************************

		************************
		*Divide training sample in 5 groups
		************************
		
			*we use a separate cross-validation split for the ensemble, 
			*If we used the same cross-validation split as for cross-validated tuning, 
			*we would give an advantage to those algorithms with many or high-dimensional tuning parameter values
			set seed 5678
			gen random_en = uniform()
			sort holdout random_en
	
			count if train==1
			local size2 = `r(N)'/5

			qui generate foldc=0 if train==1
			qui replace foldc=1 if _n>`size2' & train==1
			qui replace foldc=2 if _n>`size2'*2 & train==1
			qui replace foldc=3 if _n>`size2'*3 & train==1
			qui replace foldc=4 if _n>`size2'*4 & train==1

			tab foldc, gen(fold)
			*tab foldc train, m
			*holdout sample not in  the 5 folds, as expected
			*tab foldc holdout, m

		************************
		*Folds
		************************
		
			*Fold A
			gen byte trainA = (fold2==1 | fold3==1 | fold4==1 | fold5==1)
			gen byte testA = (fold1==1)
		
			*FoldB
			gen byte trainB = (fold1==1| fold3==1 | fold4==1 | fold5==1)
			gen byte testB = (fold2==1)
			
			*FoldC
			gen byte trainC = (fold1==1 | fold2==1| fold4==1 | fold5==1)
			gen byte testC = (fold3==1)
			
			*Fold D
			gen byte trainD = (fold1==1 | fold2==1 | fold3==1| fold5==1)
			gen byte testD = (fold4==1)
			
			*Fold E
			gen byte trainE = (fold1==1 | fold2==1 | fold3==1 | fold4==1)
			gen byte testE = (fold5==1)
		
	**************************************
	*Drop predictors that are always zero
	**************************************
	
		unab pred0: p_*
		global PRED0	`pred0'
		
		foreach var of global PRED0 {
			qui sum `var' if train==1 
			if r(mean)==0 & r(sd)==0 {
				di "`var'"
				drop `var'
			}	
		}

		*FOR GROUP 1 AND 3. NOTE: FOR THE MAIN TABLE, DO NOT INCLUDE BUSINESS SCORES IN THE INPUTS
		*drop b_round1 since it is always one (all winners passed the first round)
		*drop b_round1. Not necessary since not generated in new version data. 
		*all 1 since we have dropped RD sample with no business scores
		
		*FOR GROUP 2 AND 4
		*drop also missing variable indicators related to these variables (only for bussiness name, the others were created from tab, gen so they are fine)
		*already dropped in replication file
		*drop p_bussn_m p_bussnb_m p_bussnc_m p_mask_m p_mgive_m 
		*drop b_totaw_m
		
	**************************************
	*Global
	**************************************
	
		unab individualpredictors: p_*
		global PREDICTOR 	`individualpredictors' existing

		*Note: since we dropped the RD sample without business scores, there are no missings for these variables
		global BSCORE		b_maskill b_artic b_cap b_finsour b_finsust b_finviab b_jobcreate b_risk b_time b_unders 
		
		global FAFCHAMPS	p_selfconfw1 p_selfconfw1_m ///
							p_motivation /*p_motivation_m never missing*/ ///
							p_satisf p_satisf_m ///
							p_satexp  p_satexp_m ///
							p_abilityfw p_abilityfw_m ///
							p_creditfw p_creditfw_m ///
							p_retail 
							
		global REGION		p_region1 p_region2 p_region3 p_region4 p_region5
		global DEMOG		p_female ///
							p_age /*p_age_m never missing*/
		global COMMON		$REGION $DEMOG ///
							p_manuf p_sector1 /*missing sector*/
		
	**************************************
	*Keep only relevant variables
	**************************************		
		
		keep uid group o_* $PREDICTOR random random_en train* test* holdout b_* b_totaw b_totaw_m
		qui compress
		*count predictors
		unab countvars : $PREDICTOR
		di `: word count `countvars''
		
		*Following Mullainathan & Spiess (2017), do not drop collinear variables
		*qui _rmcoll $PREDICTOR, forcedrop
		*global PREDICTOR `r(varlist)'
				
	**************************************
	*Scale Regressors
	**************************************	
	
		*get min and max from the train sample only (as stressed in Mullainathan & Spiess, 2017)
		foreach var of global PREDICTOR {
			*Note: this do not change dummy variables: you subtract zero and divide by 1
			qui sum `var' if train==1
			qui replace `var' = (`var'-r(min))/(r(max)-r(min))
			}
				
	**************************************
	* Check no missing values
	**************************************	
	
		*check all have same number of obs as depvar
		qui sum $depvar if train==1
		local Ncheck = r(N)
		
		foreach var of global PREDICTOR {
			qui sum `var' if train==1
			if r(N)!=`Ncheck' {
				di "`var'"
				_exit
				}
		}	

	**************************************
	* Programs used in ensemble
	**************************************	
	
		capture program drop myclean
		program myclean
			sum yhat
			*check if some observations below min depvar (usually 0). If so, impute them.
			qui sum $depvar if train`1'==1
			replace yhat = r(min) if yhat<r(min)
			*check for extremely abnormal predictions
			qui sum $depvar if train`1'==1
			local yhatmax = 2*r(max)
			sum yhat
			if r(max)>`yhatmax' {
					count if yhat>`yhatmax'
					replace yhat = `yhatmax' if yhat>`yhatmax'
					sum yhat
					di ""
					di "NOTE: Prediction outliers imputed"
					di ""
				}	
		end
						
		capture program drop mymse
		program mymse
			*gen residual^2
			qui gen residual2 = ($depvar - yhat)^2
			*Hold-out sample
			qui egen SSRholdout = total(residual2) if !missing($depvar) & holdout==1
			qui sum SSRholdout
			local ssr_holdout = r(mean)
			*MSE
			qui count if !missing($depvar) & holdout==1
			local holdout_n = r(N)
			local holdout_mse= `ssr_holdout' / `holdout_n'
			di "MSE in holdout sample is " `holdout_mse'		
		end
			
		capture program drop myr2
		program myr2
			qui corr $depvar yhat if holdout==1
			local holdout_r2 = r(rho)^2
			di "The holdout R^2 is " `holdout_r2'	
		end
			
		capture program drop bootmse
		program bootmse, rclass
			qui egen SSR = total(residual2)
			qui sum SSR
			local ssr = r(mean)
			qui count 
			local nobs = r(N)
			return scalar mse= `ssr' / `nobs'
			qui drop SSR
			macro drop ssr nobs
		end
			
*exit	
		
****************************************************************************************************************************************************************		
****************************************************************************************************************************************************************		
****************************************************************************************************************************************************************		

**************************************
*Step 1: for every fold, fit each of the algorithms that make up the ensemble on all other folds, and predict on the current fold.
**************************************

	**************************************
	* LASSO
	**************************************	

		gen yhatLASSO = .
		foreach fold in A B C D E {			
			*LASSO
			di "Fold:`fold'"
			sort random_en, stable
			qui lassoregress $depvar $PREDICTOR if train`fold'==1, lambda(`bestLambda')			
			*prediction
			qui predict yhat
			*clean data
			myclean	`fold'		
			*save prediction for test sample
			replace yhatLASSO = yhat if test`fold'==1			
			*drop variables
			qui drop yhat 
		}
		
	**************************************
	* SVM
	**************************************	

		gen yhatSVM = .
		foreach fold in A B C D E {			
			*SVM
			di "Fold:`fold'"
			sort random_en, stable
			svmachines $depvar $PREDICTOR if train`fold'==1, c(`bestC') gamma(`bestGamma') type(svr) kernel(rbf) eps(`bestEps')			
			*prediction
			qui predict yhat
			*clean data
			myclean	`fold'				
			*save prediction for test sample
			replace yhatSVM = yhat if test`fold'==1			
			*drop variables
			qui drop yhat 
		}
		
	**************************************
	* Boosting
	**************************************	

		capture program boost_plugin, plugin using("C:\ado\plus\b\boost64.dll")
		gen yhatBoost = .
		foreach fold in A B C D E {			
			*Boosting
			di "Fold:`fold'"
			sort random_en, stable
			qui boost $depvar $PREDICTOR if train`fold'==1, ///
				distribution(normal) maxiter(`bestIter') predict(yhat) shrink(`bestShrink') bag(0.5) trainfraction(1) interaction(`bestInter') seed(1234) 
			*clean data
			myclean	`fold'				
			*save prediction for test sample
			replace yhatBoost = yhat if test`fold'==1			
			*drop variables
			qui drop yhat 
		}

	**************************************
	* Raven
	**************************************	
			
		gen yhatRav = .
		foreach fold in A B C D E {			
			*OLS
			di "Fold:`fold'"
			sort random_en, stable
			qui regress $depvar p_raven p_raven_m existing if train`fold'==1
			*prediction
			qui predict yhat
			*clean data
			myclean	`fold'		
			*save prediction for test sample
			replace yhatRav= yhat if test`fold'==1			
			*drop variables
			qui drop yhat 
		}

	**************************************
	* Judges
	**************************************	
		
		gen yhatJud = .
		foreach fold in A B C D E {			
			*OLS
			di "Fold:`fold'"
			sort random_en, stable
			
			*non-winners
			qui regress $depvar $BSCORE existing if train`fold'==1
*			*winners
			*qui regress $depvar $BSCORE existing p_group1 b_totaw b_totaw_m if train`fold'==1
			
			*prediction
			qui predict yhat
			*clean data
			myclean	`fold'		
			*save prediction for test sample
			replace yhatJud= yhat if test`fold'==1			
			*drop variables
			qui drop yhat 
		}	
		
	**************************************
	* Fafchamps-Woodruff
	**************************************	
			
		gen yhatFaWo = .
		foreach fold in A B C D E {			
			*OLS
			di "Fold:`fold'"
			sort random_en, stable
			qui regress $depvar $COMMON $FAFCHAMPS existing if train`fold'==1
			*prediction
			qui predict yhat
			*clean data
			myclean	`fold'		
			*save prediction for test sample
			replace yhatFaWo= yhat if test`fold'==1			
			*drop variables
			qui drop yhat 
		}		
		
**************************************
*Step 2: run a linear regression of the outcome on these predicted values in the full training sample, and store the resulting linear model.
**************************************

		*we do not use the in-sample fitted values in the training sample to choose ensemble weights, 
		*as this would produce larger weights for algorithms that overfit more

	***
	*Ensemble 1: LASSO +  SVM + Boost
	***

		regress $depvar yhatLASSO yhatSVM yhatBoost if train==1
		local cons_e1=_b[_cons]
		local b1_e1= _b[yhatLASSO]
		local b2_e1= _b[yhatSVM]
		local b3_e1= _b[yhatBoost]
		
			/*
			*check predictions manually and automatically are the same
			predict yhat_r2
			gen yhat_r2b = `cons_e1' + `b1_e1'*yhatLASSO + `b2_e1'*yhatSVM + `b3_e1'*yhatBoost
			count if yhat_r2!=yhat_r2b
			*0
			*/
			
	***
	*Ensemble 2: ML + Raven
	***

		regress $depvar yhatLASSO yhatSVM yhatBoost yhatRav if train==1
		local cons_e2=_b[_cons]
		local b1_e2= _b[yhatLASSO]
		local b2_e2= _b[yhatSVM]
		local b3_e2= _b[yhatBoost]
		local b4_e2= _b[yhatRav]
		
	***
	*Ensemble 3: ML + Judges
	***

		regress $depvar yhatLASSO yhatSVM yhatBoost yhatJud if train==1
		local cons_e3=_b[_cons]
		local b1_e3= _b[yhatLASSO]
		local b2_e3= _b[yhatSVM]
		local b3_e3= _b[yhatBoost]
		local b4_e3= _b[yhatJud]		

	***
	*Ensemble 4: ML + FaWo
	***

		regress $depvar yhatLASSO yhatSVM yhatBoost yhatFaWo if train==1
		local cons_e4=_b[_cons]
		local b1_e4= _b[yhatLASSO]
		local b2_e4= _b[yhatSVM]
		local b3_e4= _b[yhatBoost]
		local b4_e4= _b[yhatFaWo]		
		
		
**************************************
*Step 3: fit each algorithm on full training sample, obtain predictions for each algorithm on hold-out, predict with OLS applied to these fitted values.
**************************************

	**************************************
	*LASSO
	**************************************
	
		*LASSO
		sort random, stable
		qui lassoregress $depvar $PREDICTOR if train==1, lambda(`bestLambda')		
		*predict
		predict yhat
		*clean data
		myclean	`fold'
		qui gen yhatLASSOb = yhat
		qui drop yhat			
		
			/*
			*check same result as main table
			gen yhat = yhatLASSOb
			mymse
			myr2
			preserve
				keep if holdout==1 
				bootstrap r(mse), reps(1000) seed(1111): bootmse
			restore
			qui drop yhat residual2 SSRholdout
			qui macro drop ssr_holdout holdout_n holdout_mse holdout_r2
			*/
			
	**************************************
	*SVM
	**************************************
	
		sort random, stable
		qui svmachines $depvar $PREDICTOR if train==1, c(`bestC') gamma(`bestGamma') type(svr) kernel(rbf) eps(`bestEps')
		*predict
		predict yhat
		*clean data
		myclean	`fold'
		qui gen yhatSVMb = yhat
		qui drop yhat			

	**************************************
	*Boost
	**************************************
	
		*Boost
		sort random, stable
		qui boost $depvar $PREDICTOR if train==1, ///
				distribution(normal) maxiter(`bestIter') predict(yhat) shrink(`bestShrink') bag(0.5) trainfraction(1) interaction(`bestInter') seed(1234) 
		*clean data
		myclean	`fold'
		qui gen yhatBoostb = yhat
		qui drop yhat			
			
	**************************************
	*Raven
	**************************************
	
		*OLS
		sort random, stable
		qui regress $depvar p_raven p_raven_m existing if train==1
		predict yhat
		*clean data
		myclean	`fold'
		qui gen yhatRavb = yhat
		qui drop yhat			

	**************************************
	*Judges
	**************************************
	
		*OLS
		sort random, stable

		*non-winners
		qui regress $depvar $BSCORE existing if train==1
*		*winners
		*qui regress $depvar $BSCORE existing p_group1 b_totaw b_totaw_m if train==1
		
		predict yhat
		*clean data
		myclean	`fold'
		qui gen yhatJudb = yhat
		qui drop yhat			

	**************************************
	*Fafchamps-Woodruff
	**************************************
	
		*OLS
		sort random, stable
		qui regress $depvar $COMMON $FAFCHAMPS existing if train==1
		predict yhat
		*clean data
		myclean	`fold'
		qui gen yhatFaWob = yhat
		qui drop yhat			
		
	**************************************
	*predict with the linear model obtained in the previous step from these fitted values
	**************************************
	
		***
		*Ensemble 1: LASSO +  SVM + Boost
		***
		
			gen yhat = `cons_e1' + `b1_e1'*yhatLASSOb + `b2_e1'*yhatSVMb + `b3_e1'*yhatBoostb
			*MSE
			mymse
			*R2
			myr2
			*boostrap out-of-sample performance			
			preserve
				keep if holdout==1 
				bootstrap r(mse), reps(1000) seed(1111): bootmse
			restore
			qui drop yhat residual2 SSRholdout
			
				/*
				*check same results with main dofiles
				*LASSO
				gen yhat = yhatLASSOb
				mymse 
				myr2
				qui drop yhat residual2 SSRholdout
				qui macro drop ssr_holdout holdout_n holdout_mse holdout_r2
				*SVM
				gen yhat = yhatSVMb
				mymse 
				myr2
				qui drop yhat residual2 SSRholdout
				qui macro drop ssr_holdout holdout_n holdout_mse holdout_r2
				*Boost
				gen yhat = yhatBoostb
				mymse 
				myr2
				qui drop yhat residual2 SSRholdout
				qui macro drop ssr_holdout holdout_n holdout_mse holdout_r2
				*/

		***
		*Ensemble 2: ML + Raven
		***
		
			gen yhat = `cons_e2' + `b1_e2'*yhatLASSOb + `b2_e2'*yhatSVMb + `b3_e2'*yhatBoostb + `b4_e2'*yhatRavb
			*MSE
			mymse
			*R2
			myr2
			*boostrap out-of-sample performance			
			preserve
				keep if holdout==1 
				bootstrap r(mse), reps(1000) seed(1111): bootmse
			restore
			qui drop yhat residual2 SSRholdout

		***
		*Ensemble 3: ML + Judges
		***
		
			gen yhat = `cons_e3' + `b1_e3'*yhatLASSOb + `b2_e3'*yhatSVMb + `b3_e3'*yhatBoostb + `b4_e3'*yhatJudb
			*MSE
			mymse
			*R2
			myr2
			*boostrap out-of-sample performance			
			preserve
				keep if holdout==1 
				bootstrap r(mse), reps(1000) seed(1111): bootmse
			restore
			qui drop yhat residual2 SSRholdout		

		***
		*Ensemble 4: ML + FaWo
		***
		
			gen yhat = `cons_e4' + `b1_e4'*yhatLASSOb + `b2_e4'*yhatSVMb + `b3_e4'*yhatBoostb + `b4_e4'*yhatFaWob
			*MSE
			mymse
			*R2
			myr2
			*boostrap out-of-sample performance			
			preserve
				keep if holdout==1 
				bootstrap r(mse), reps(1000) seed(1111): bootmse
			restore
			qui drop yhat residual2 SSRholdout		
			
log close

exit
