**********************************
**********************************
** ML Nigeria
** B7 - Portfolio Simulation
**********************************
**********************************

/*
	READ ME
	set directory: line 26, 432
	Turn on/off:
		groups: lines 43-44
	For the ML predictions:
		run until the exit command 421-556
		then run the desired lines, e.g. 566-573 for LASSO profits non-winners
		
*/

capture log close
clear all
set more off
set matsize 11000
version 14.1 

*set directory
cd 
*load data
use "Data\MLNigeria_masterdata.dta", clear

*Note:
*Firms in control group who actually recived award (9 firms) dropped during data cleaning (dofile A1)
*Firms without business plan scores from RD sample dropped during data cleaning (dofile A1)
*Firms disqualified dropped during data cleaning (dofile A1)
		
**************************************
*Data Preparation
**************************************

	**************************************
	*Keep only relevant groups
	**************************************

		keep if (group==1|group==3)
		*keep if (group==2|group==4)
		
		global depvar o_t_profits_IHS
		keep if $depvar!=.

	**************************************
	*Global
	**************************************
	
		*Note: since we dropped the RD sample without business scores, there are no missings for these variables
		global BSCORE		b_maskill b_artic b_cap b_finsour b_finsust b_finviab b_jobcreate b_risk b_time b_unders 
		
		*Group 1&3: control for winning status (p_group1) only when using business plan scores as predictors
		*Group 2&4: Do NOT control for RD group
		*global GROUP		existing /*p_group1*/ 			
		global GROUP		existing /*p_group4*/	

		global MCKENZIE 	p_higheduc p_educ_m ///
							p_workabroad /*p_workabroad_m never missing*/ ///
							p_lot901 /*p_lot905 never missing*/ ///
							p_wealthindex /*p_wealthindex_m never missing*/ ///
							p_grit p_grit_m ///
							p_ability p_ability_m ///
							p_sector31 /*agriculture*/ p_sector19 /*IT*/
							
		global FAFCHAMPS	p_selfconfw1 p_selfconfw1_m ///
							p_motivation /*p_motivation_m never missing*/ ///
							p_satisf p_satisf_m ///
							p_satexp  p_satexp_m ///
							p_abilityfw p_abilityfw_m ///
							p_creditfw p_creditfw_m ///
							p_retail 
							
		global REGION		p_region1 p_region2 p_region3 p_region4 p_region5
		global DEMOG		p_female ///
							p_age /*p_age_m never missing*/
		global COMMON		$REGION $DEMOG ///
							p_manuf p_sector1 /*missing sector*/
		
	**************************************
	*Divide sample in trainig and hold-out samples
	**************************************	
	
		*sort data in random order
		set seed 1234
		gen random = uniform()
		sort random
		
		*Divide in 2 groups
		local size = _N * 0.8	
		qui gen train= (_n<=`size')
		qui gen holdout = (_n>`size')
		*tab train holdout, m col
			
	**************************************
	*Post-estimation programs
	**************************************
	
		capture program drop bootmse
		program bootmse, rclass
			qui egen SSR = total(residual2)
			qui sum SSR
			local ssr = r(mean)
			qui count 
			local nobs = r(N)
			return scalar mse= `ssr' / `nobs'
			qui drop SSR
			macro drop ssr nobs
		end
		
		capture program drop pmse
		program pmse
					
			*predict yhat
			qui predict yhat
			sum yhat
			
			*check if some observations below min depvar (usually 0). If so, impute them.
			*save this as a different variable, since you want to do the ranking without imputations
			gen yhat2 = yhat
			qui sum $depvar if train==1
			replace yhat2 = r(min) if yhat2<r(min)
			*check for extremely abnormal predictions
			qui sum $depvar if train==1
			local yhatmax = 2*r(max)
			sum yhat2
			if r(max)>`yhatmax' {
				replace yhat2 = `yhatmax' if yhat2>`yhatmax'
				sum yhat2
				di ""
				di "NOTE: Prediction outliers imputed"
				di ""
				}						
			
			*gen residual^2
			qui gen residual2 = ($depvar - yhat2)^2
			
			*number observations
			qui count if train==1 & $depvar!=.
			local train_n = r(N)
			qui count if holdout==1 & $depvar!=.
			local holdout_n = r(N)
				
			*train MSE	
			qui egen train_SSR = total(residual2) if !missing($depvar) & train==1
			qui sum train_SSR
			local train_mse= r(mean) / `train_n'
			di "The train MSE is " round(`train_mse', 0.01)

			*train r2
			qui corr $depvar yhat2 if train==1
			local train_r2 = r(rho)^2
			di "The train R^2 is " round(`train_r2'*100, 0.1)
			
			*holdout r2
			qui corr $depvar yhat2 if holdout==1
			local holdout_r2 = r(rho)^2
			di "The holdout R^2 is " round(`holdout_r2'*100, 0.1)
			
			*holdout MSE	
			qui egen holdout_SSR = total(residual2) if !missing($depvar) & holdout==1
			qui sum holdout_SSR
			local holdout_mse= r(mean) / `holdout_n'
			di "The holdout MSE is " round(`holdout_mse', 0.01)
			
			preserve
				keep if holdout==1
				*check thta bootmse provides the same average mse
					*bootmse
					*return list
				bootstrap r(mse), reps(1000) seed(1111): bootmse
			restore
			
			drop yhat2 residual2 train_SSR holdout_SSR
			macro drop train_n holdout_n train_mse holdout_r2 holdout_mse train_r2
		
		end	
		
		capture program drop boottprof
		program boottprof, rclass
			qui sort yhat, stable
			qui gen selected = 1 in -21/L
			qui egen tottop = total(o_t_profits) if selected==1
			qui sum tottop
			return scalar tprof = r(mean)
			*even if profits are always positive for non-winners, the 95% CI might include negative values if the mean is close enough to zero and the SD large enough				
				
				*check o_t_profits always positive
				*qui sum o_t_profits
				*return scalar tprof = r(mean)
				*return scalar tprof = r(min)
			
				*check selected always 21
				*qui sum selected
				*return scalar tprof = r(N)
				
			qui drop selected tottop 
		end
			
		capture program drop totprof
		program totprof
		
			*sort by profits in test sample
			qui sort holdout yhat, stable
			
			*size holdout sample in 2&4: 212
			*size holdout sample in 1&3: 208
			*get top 21 firms (around 10% test sample)
			qui gen selected = 1 in -21/L
			
			*sum profits (levels) for these 21 firms
			qui egen tottop = total(o_t_profits) if selected==1
			sum tottop
			di "The holdout total profit is " r(mean)
			qui drop selected tottop
			
			*boostrap performance
			preserve
				keep if holdout==1
				*check thta bootaccu provides the same accuracy
					*boottprof
					*return list
				*check min profits should always be zero for non-winners
				*bootstrap r(min), reps(1000) seed(1111) cformat(%9.0f): sum o_t_profits	
				bootstrap r(tprof), reps(1000) seed(1111) cformat(%9.0f): boottprof
			restore
			
		end	
						
exit		
		
**************************************
* Prediction
**************************************
		
	**************************************
	*Constant
	**************************************

		regress $depvar if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof
	
	**************************************
	*Initial Score
	**************************************
	
		*no missings
		*codebook totalmark
		
		*non-winners
		regress $depvar b_totmarkf $GROUP if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof
		
		*winners
		regress $depvar b_totmarkf $GROUP b_totaw b_totaw_m if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof
		
	**************************************
	*Business Plan Total Score
	**************************************
	
		*no missings
		*codebook totalmarks
		
		*non-winners
		regress $depvar b_totmark $GROUP if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof

		*winners
		regress $depvar b_totmark $GROUP p_group1 b_totaw b_totaw_m if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof
						
	**************************************
	*Business Plan Score Components
	**************************************
	
		*Non-winners
		regress $depvar $BSCORE $GROUP if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof
		
		*Winners
		regress $depvar $BSCORE $GROUP p_group1 b_totaw b_totaw_m if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof

	**************************************
	*Gender
	**************************************
	
		*no missings
		*codebook sex
		regress $depvar p_female $GROUP if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof
		
	**************************************
	*Age
	**************************************
	
		*no missings
		*codebook age
		regress $depvar p_age $GROUP if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof

	**************************************
	*Necessity Entrepreneurship
	**************************************
	
		*no missings
		*codebook rate_employment
		regress $depvar p_needjob $GROUP if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof
		
	**************************************
	*Grit
	**************************************
	
		regress $depvar p_grit p_grit_m $GROUP if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof

	**************************************
	*Backward Digit-span Recall
	**************************************
	
		regress $depvar p_digitspan p_digitspan_m $GROUP if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof
		
	**************************************
	*Raven Test
	**************************************

		regress $depvar p_raven p_raven_m $GROUP if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof
		
	**************************************
	*Registration Time
	**************************************
	
		*no missings
		*codebook reg_time
		regress $depvar p_lateentrant $GROUP if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof
								
	**************************************
	*McKenzie
	**************************************
		
		regress $depvar $COMMON $MCKENZIE $GROUP if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof
				
	**************************************
	*Fafchamps-Woodruff
	**************************************
							
		regress $depvar $COMMON $FAFCHAMPS $GROUP if train==1
		*check if same results as main tables
		pmse
		*tail profits
		totprof
		
		*with Business scores
		*Non-winners
		regress $depvar $COMMON $FAFCHAMPS $GROUP b_totmark if train==1
		pmse
		totprof
		*Winners
		regress $depvar $COMMON $FAFCHAMPS $GROUP b_totmark p_group1 b_totaw b_totaw_m if train==1
		pmse
		totprof
		
*****************************************************************************************************************************************************		
*****************************************************************************************************************************************************		
*****************************************************************************************************************************************************		

**************************************
*Machine Learning
**************************************

	capture log close
	clear all
	set more off
	set matsize 11000
	version 14.1 

	*set directory
	cd 
	*Note: the datasets used in this dofile were saved when running the dofiles B1, B2, B3
	
	**************************************
	*Post-estimation programs (do not need to predict yhat)
	**************************************
	
		capture program drop bootmse
		program bootmse, rclass
			qui egen SSR = total(residual2)
			qui sum SSR
			local ssr = r(mean)
			qui count 
			local nobs = r(N)
			return scalar mse= `ssr' / `nobs'
			qui drop SSR
			macro drop ssr nobs
		end
		
		capture program drop pmse
		program pmse
		
			drop residual2
								
			*check if some observations below min depvar (usually 0). If so, impute them.
			*save this as a different variable, since you want to do the ranking without imputations
			gen yhat2 = yhat
			qui sum $depvar if train==1
			replace yhat2 = r(min) if yhat2<r(min)
			*check for extremely abnormal predictions
			qui sum $depvar if train==1
			local yhatmax = 2*r(max)
			sum yhat2
			if r(max)>`yhatmax' {
				replace yhat2 = `yhatmax' if yhat2>`yhatmax'
				sum yhat2
				di ""
				di "NOTE: Prediction outliers imputed"
				di ""
				}						
			
			*gen residual^2
			qui gen residual2 = ($depvar - yhat2)^2
			
			*number observations
			qui count if train==1 & $depvar!=.
			local train_n = r(N)
			qui count if holdout==1 & $depvar!=.
			local holdout_n = r(N)
				
			*train MSE	
			qui egen train_SSR = total(residual2) if !missing($depvar) & train==1
			qui sum train_SSR
			local train_mse= r(mean) / `train_n'
			di "The train MSE is " round(`train_mse', 0.01)

			*train r2
			qui corr $depvar yhat2 if train==1
			local train_r2 = r(rho)^2
			di "The train R^2 is " round(`train_r2'*100, 0.1)
			
			*holdout r2
			qui corr $depvar yhat2 if holdout==1
			local holdout_r2 = r(rho)^2
			di "The holdout R^2 is " round(`holdout_r2'*100, 0.1)
			
			*holdout MSE	
			qui egen holdout_SSR = total(residual2) if !missing($depvar) & holdout==1
			qui sum holdout_SSR
			local holdout_mse= r(mean) / `holdout_n'
			di "The holdout MSE is " round(`holdout_mse', 0.01)
			
			preserve
				keep if holdout==1
				*check thta bootmse provides the same average mse
					*bootmse
					*return list
				bootstrap r(mse), reps(1000) seed(1111): bootmse
			restore
			
			drop yhat2 residual2 train_SSR holdout_SSR
			macro drop train_n holdout_n train_mse holdout_r2 holdout_mse train_r2
		
		end	
		
		capture program drop boottprof
		program boottprof, rclass
			qui sort yhat, stable
			qui gen selected = 1 in -21/L
			qui egen tottop = total(o_t_profits) if selected==1
			qui sum tottop
			return scalar tprof= r(mean)
			qui drop selected tottop
		end
		
	
		capture program drop totprof
		program totprof
		
			*sort by profits in test sample
			qui sort holdout yhat, stable
			
			*size holdout sample in 2&4: 212
			*size holdout sample in 1&3: 208
			*get top 21 firms (around 10% test sample)
			qui gen selected = 1 in -21/L
			
			*sum profits (levels) for these 21 firms
			qui egen tottop = total(o_t_profits) if selected==1
			sum tottop
			di "The holdout total profit is " r(mean)
			qui drop selected tottop
			
			*boostrap performance
			preserve
				keep if holdout==1
				*check thta bootaccu provides the same accuracy
					*boottprof
					*return list
				bootstrap r(tprof), reps(1000) seed(1111) cformat(%9.0f): boottprof
			restore
			
		end	
						
exit		
	
**************************************
*Profits
**************************************

	*********
	*P24
	*********
		
		*LASSO
		use "LASSOProf24.dta", clear
		global depvar o_t_profits_IHS
		rename LASSO_Prof24 yhat
		*check if same results as main tables
		pmse
		*tail profits
		totprof

		*SVM
		use "SVMProf24.dta", clear
		global depvar o_t_profits_IHS
		rename SVM_Prof24 yhat
		*check if same results as main tables
		pmse
		*tail profits
		totprof
		
		*Boosting
		use "BoostProf24.dta", clear
		global depvar o_t_profits_IHS
		rename Boost_Prof24 yhat
		*check if same results as main tables
		pmse
		*tail profits
		totprof
		
	*********
	*P13
	*********
		
		*LASSO
		use "LASSOProf13.dta", clear
		global depvar o_t_profits_IHS
		rename LASSO_Prof13 yhat
		*check if same results as main tables
		pmse
		*tail profits
		totprof

		*SVM
		use "SVMProf13.dta", clear
		global depvar o_t_profits_IHS
		rename SVM_Prof13 yhat
		*check if same results as main tables
		pmse
		*tail profits
		totprof
		
		*Boosting
		use "BoostProf13.dta", clear
		global depvar o_t_profits_IHS
		rename Boost_Prof13 yhat
		*check if same results as main tables
		pmse
		*tail profits
		totprof

exit
